"""Base class for all objects requiring a valid names for all engines."""
from pydantic import BaseModel, Field, ConfigDict
from typing import Union


class NoExtraBaseModel(BaseModel):
    """Base class for all objects that are not extensible with additional keys.

    This effectively includes all objects except for the Properties classes
    that are assigned to geometry objects.
    """

    model_config = ConfigDict(extra='forbid')


class IDdBaseModel(NoExtraBaseModel):
    """Base class for all objects requiring a identifiers acceptable for all engines."""

    identifier: str = Field(
        ...,
        pattern=r'^[.A-Za-z0-9_-]+$',
        min_length=1,
        max_length=100,
        description='Text string for a unique object ID. This identifier remains '
        'constant as the object is mutated, copied, and serialized to different '
        'formats (eg. dict, idf, rad). This identifier is also used to reference '
        'the object across a Model. It must be < 100 characters and not contain '
        'any spaces or special characters.'
    )

    display_name: Union[str, None] = Field(
        default=None,
        description='Display name of the object with no character restrictions.'
    )

    user_data: Union[dict, None] = Field(
        default=None,
        description='Optional dictionary of user data associated with the object.'
        'All keys and values of this dictionary should be of a standard data '
        'type to ensure correct serialization of the object (eg. str, float, '
        'int, list).'
    )



"""Properties Schema"""
from pydantic import Field
from typing import List, Union, Literal

from .modifierset import ModifierSet, ModifierSetAbridged
from .modifier import Plastic, Glass, BSDF, Glow, Light, Trans, Metal, Void, Mirror
from .global_modifierset import GlobalModifierSet
from .asset import SensorGrid, View
from .._base import NoExtraBaseModel

from honeybee_schema.radiance.state import RadianceShadeStateAbridged, \
    RadianceSubFaceStateAbridged


class _PropertiesBaseAbridged(NoExtraBaseModel):
    """Base class of Abridged Radiance Properties."""

    modifier: str = Field(
        default=None,
        description='A string for a Honeybee Radiance Modifier (default: None).'
    )

    modifier_blk: str = Field(
        default=None,
        description='A string for a Honeybee Radiance Modifier to be used '
                    'in direct solar simulations and in isolation studies (assessing'
                    'the contribution of individual objects) (default: None).'
    )


class ShadeMeshRadiancePropertiesAbridged(_PropertiesBaseAbridged):
    """Radiance Properties for Honeybee ShadeMesh Abridged."""

    type: Literal['ShadeMeshRadiancePropertiesAbridged'] = 'ShadeMeshRadiancePropertiesAbridged'


class ApertureRadiancePropertiesAbridged(_PropertiesBaseAbridged):
    """Radiance Properties for Honeybee Aperture Abridged."""

    type: Literal['ApertureRadiancePropertiesAbridged'] = 'ApertureRadiancePropertiesAbridged'

    dynamic_group_identifier: str = Field(
        default=None,
        description="An optional string to note the dynamic group ' \
            'to which the Aperture is a part of. Apertures sharing the same ' \
            'dynamic_group_identifier will have their states change in unison. ' \
            'If None, the Aperture is assumed to be static. (default: None)."
    )

    states: List[RadianceSubFaceStateAbridged] = Field(
        default=None,
        description="An optional list of abridged states (default: None)."
    )


class DoorRadiancePropertiesAbridged(_PropertiesBaseAbridged):
    """Radiance Properties for Honeybee Door Abridged."""

    type: Literal['DoorRadiancePropertiesAbridged'] = 'DoorRadiancePropertiesAbridged'

    dynamic_group_identifier: str = Field(
        default=None,
        description="An optional string to note the dynamic group ' \
            'to which the Door is a part of. Doors sharing the same ' \
            'dynamic_group_identifier will have their states change in unison. ' \
            'If None, the Door is assumed to be static. (default: None)."
    )

    states: List[RadianceSubFaceStateAbridged] = Field(
        default=None,
        description="An optional list of abridged states (default: None)."
    )


class FaceRadiancePropertiesAbridged(_PropertiesBaseAbridged):
    """Radiance Properties for Honeybee Face Abridged."""

    type: Literal['FaceRadiancePropertiesAbridged'] = 'FaceRadiancePropertiesAbridged'


class ShadeRadiancePropertiesAbridged(_PropertiesBaseAbridged):
    """Radiance Properties for Honeybee Shade Abridged."""

    type: Literal['ShadeRadiancePropertiesAbridged'] = 'ShadeRadiancePropertiesAbridged'

    dynamic_group_identifier: str = Field(
        default=None,
        description='An optional string to note the dynamic group to which the Shade '
        'is a part of. Shades sharing the same dynamic_group_identifier will have '
        'their states change in unison. If None, the Shade is assumed to be static.'
    )

    states: List[RadianceShadeStateAbridged] = Field(
        default=None,
        description="An optional list of abridged states (default: None)."
    )


class RoomRadiancePropertiesAbridged(NoExtraBaseModel):
    """Abridged Radiance Properties for Honeybee Room."""

    type: Literal['RoomRadiancePropertiesAbridged'] = 'RoomRadiancePropertiesAbridged'

    modifier_set: str = Field(
        default=None,
        description='An identifier for a unique Room-Assigned ModifierSet '
                    '(default: None).'
    )


class ModelRadianceProperties(NoExtraBaseModel):
    """Radiance Properties for Honeybee Model."""

    type: Literal['ModelRadianceProperties'] = 'ModelRadianceProperties'

    global_modifier_set: GlobalModifierSet = Field(
        default=GlobalModifierSet(),
        description='Global Radiance modifier set.',
        json_schema_extra={'readOnly': True}
    )

    modifiers: List[Union[Plastic, Glass, BSDF, Glow, Light, Trans, Metal, Void, Mirror]] = Field(
        default=None,
        description='A list of all unique modifiers in the model. '
                    'This includes modifiers across all Faces, Apertures, Doors, '
                    'Shades, Room ModifierSets, and the global_modifier_set.'
    )

    modifier_sets: List[Union[ModifierSet, ModifierSetAbridged]] = Field(
        default=None,
        description='A list of all unique Room-Assigned ModifierSets in the Model.'
    )

    sensor_grids: List[SensorGrid] = Field(
        default=None,
        description='An array of SensorGrids that are associated with the model.'
    )

    views: List[View] = Field(
        default=None,
        description='An array of Views that are associated with the model.'
    )



"""ConstructionSet Schema"""
from pydantic import Field
from typing import Union, Literal

from .._base import NoExtraBaseModel
from ._base import IDdEnergyBaseModel
from .construction import OpaqueConstruction, WindowConstruction, ShadeConstruction, \
    AirBoundaryConstruction, WindowConstructionShade, WindowConstructionDynamic


class _FaceSubSetAbridged(NoExtraBaseModel):
    """A set of constructions for wall, floor, or roof assemblies."""

    interior_construction: Union[str, None] = Field(
        default=None,
        min_length=1,
        max_length=100,
        description='Identifier for an OpaqueConstruction for faces with a Surface or '
        'Adiabatic boundary condition.'
    )

    exterior_construction: Union[str, None] = Field(
        default=None,
        min_length=1,
        max_length=100,
        description='Identifier for an OpaqueConstruction for faces with an Outdoors '
        'boundary condition.'
    )

    ground_construction: Union[str, None] = Field(
        default=None,
        min_length=1,
        max_length=100,
        description='Identifier for an OpaqueConstruction for faces with a Ground '
        'boundary condition.'
    )


class WallConstructionSetAbridged(_FaceSubSetAbridged):
    """A set of constructions for wall assemblies."""

    type: Literal['WallConstructionSetAbridged'] = 'WallConstructionSetAbridged'


class FloorConstructionSetAbridged(_FaceSubSetAbridged):
    """A set of constructions for floor assemblies."""

    type: Literal['FloorConstructionSetAbridged'] = 'FloorConstructionSetAbridged'


class RoofCeilingConstructionSetAbridged(_FaceSubSetAbridged):
    """A set of constructions for roof and ceiling assemblies."""

    type: Literal['RoofCeilingConstructionSetAbridged'] = 'RoofCeilingConstructionSetAbridged'


class _FaceSubSet(NoExtraBaseModel):
    """A set of constructions for wall, floor, or roof assemblies."""

    interior_construction: Union[OpaqueConstruction, None] = Field(
        default=None,
        description='An OpaqueConstruction for walls with a Surface or '
        'Adiabatic boundary condition.'
    )

    exterior_construction: Union[OpaqueConstruction, None] = Field(
        default=None,
        description='An OpaqueConstruction for walls with an Outdoors '
        'boundary condition.'
    )

    ground_construction: Union[OpaqueConstruction, None] = Field(
        default=None,
        description='An OpaqueConstruction for walls with a Ground '
        'boundary condition.'
    )


class WallConstructionSet(_FaceSubSet):
    """A set of constructions for wall assemblies."""

    type: Literal['WallConstructionSet'] = 'WallConstructionSet'


class FloorConstructionSet(_FaceSubSet):
    """A set of constructions for floor assemblies."""

    type: Literal['FloorConstructionSet'] = 'FloorConstructionSet'


class RoofCeilingConstructionSet(_FaceSubSet):
    """A set of constructions for roof and ceiling assemblies."""

    type: Literal['RoofCeilingConstructionSet'] = 'RoofCeilingConstructionSet'


class ApertureConstructionSetAbridged(NoExtraBaseModel):
    """A set of constructions for aperture assemblies."""

    type: Literal['ApertureConstructionSetAbridged'] = 'ApertureConstructionSetAbridged'

    interior_construction: Union[str, None] = Field(
        default=None,
        min_length=1,
        max_length=100,
        description='Identifier for a WindowConstruction for all apertures with a '
        'Surface boundary condition.'
    )

    window_construction: Union[str, None] = Field(
        default=None,
        min_length=1,
        max_length=100,
        description='Identifier for a WindowConstruction for apertures with an '
        'Outdoors boundary condition, False is_operable property, and a Wall '
        'face type for their parent face.'
    )

    skylight_construction: Union[str, None] = Field(
        default=None,
        min_length=1,
        max_length=100,
        description='Identifier for a WindowConstruction for apertures with a Outdoors '
        'boundary condition, False is_operable property, and a RoofCeiling or '
        'Floor face type for their parent face.'
    )

    operable_construction: Union[str, None] = Field(
        default=None,
        min_length=1,
        max_length=100,
        description='Identifier for a WindowConstruction for all apertures with an '
        'Outdoors boundary condition and True is_operable property.'
    )


class ApertureConstructionSet(NoExtraBaseModel):
    """A set of constructions for aperture assemblies."""

    type: Literal['ApertureConstructionSet'] = 'ApertureConstructionSet'

    interior_construction: Union[
        WindowConstruction, WindowConstructionShade, WindowConstructionDynamic, None
    ] = Field(
        default=None, description='A WindowConstruction for all apertures with a '
        'Surface boundary condition.'
    )

    window_construction: Union[
        WindowConstruction, WindowConstructionShade, WindowConstructionDynamic, None
    ] = Field(
        default=None, description='A WindowConstruction for apertures with an '
        'Outdoors boundary condition, False is_operable property, and a Wall '
        'face type for their parent face.'
    )

    skylight_construction: Union[
        WindowConstruction, WindowConstructionShade, WindowConstructionDynamic, None
    ] = Field(
        default=None,
        description='A WindowConstruction for apertures with a Outdoors boundary '
        'condition, False is_operable property, and a RoofCeiling or '
        'Floor face type for their parent face.'
    )

    operable_construction: Union[
        WindowConstruction, WindowConstructionShade, WindowConstructionDynamic, None
    ] = Field(
        default=None,
        description='A WindowConstruction for all apertures with an '
        'Outdoors boundary condition and True is_operable property.'
    )


class DoorConstructionSetAbridged(NoExtraBaseModel):
    """A set of constructions for door assemblies."""

    type: Literal['DoorConstructionSetAbridged'] = 'DoorConstructionSetAbridged'

    interior_construction: Union[str, None] = Field(
        default=None,
        min_length=1,
        max_length=100,
        description='Identifier for an OpaqueConstruction for all opaque doors with a '
        'Surface boundary condition.'
    )

    exterior_construction: Union[str, None] = Field(
        default=None,
        min_length=1,
        max_length=100,
        description='Identifier for an OpaqueConstruction for opaque doors with an '
        'Outdoors boundary condition and a Wall face type for their parent face.'
    )

    overhead_construction: Union[str, None] = Field(
        default=None,
        min_length=1,
        max_length=100,
        description='Identifier for an OpaqueConstruction for opaque doors with an '
        'Outdoors boundary condition and a RoofCeiling or Floor type for '
        'their parent face.'
    )

    exterior_glass_construction: Union[str, None] = Field(
        default=None,
        min_length=1,
        max_length=100,
        description='Identifier for a WindowConstruction for all glass doors with an '
        'Outdoors boundary condition.'
    )

    interior_glass_construction: Union[str, None] = Field(
        default=None,
        min_length=1,
        max_length=100,
        description='Identifier for a WindowConstruction for all glass doors with a '
        'Surface boundary condition.'
    )


class DoorConstructionSet(NoExtraBaseModel):
    """A set of constructions for door assemblies."""

    type: Literal['DoorConstructionSet'] = 'DoorConstructionSet'

    interior_construction: Union[OpaqueConstruction, None] = Field(
        default=None,
        description='An OpaqueConstruction for all opaque doors with a '
        'Surface boundary condition.'
    )

    exterior_construction: Union[OpaqueConstruction, None] = Field(
        default=None,
        description='An OpaqueConstruction for opaque doors with an '
        'Outdoors boundary condition and a Wall face type for their parent face.'
    )

    overhead_construction: Union[OpaqueConstruction, None] = Field(
        default=None,
        description='An OpaqueConstruction for opaque doors with an '
        'Outdoors boundary condition and a RoofCeiling or Floor type for '
        'their parent face.'
    )

    exterior_glass_construction: Union[
        WindowConstruction, WindowConstructionShade, WindowConstructionDynamic, None
    ] = Field(
        default=None,
        description='A WindowConstruction for all glass doors with an '
        'Outdoors boundary condition.'
    )

    interior_glass_construction: Union[
        WindowConstruction, WindowConstructionShade, WindowConstructionDynamic, None
    ] = Field(
        default=None,
        description='A WindowConstruction for all glass doors with a '
        'Surface boundary condition.'
    )


class ConstructionSetAbridged(IDdEnergyBaseModel):
    """A set of constructions for different surface types and boundary conditions."""

    type: Literal['ConstructionSetAbridged'] = 'ConstructionSetAbridged'

    wall_set: Union[WallConstructionSetAbridged, None] = Field(
        default=None,
        description='A WallConstructionSetAbridged object for this ConstructionSet.'
    )

    floor_set: Union[FloorConstructionSetAbridged, None] = Field(
        default=None,
        description='A FloorConstructionSetAbridged object for this ConstructionSet.'
    )

    roof_ceiling_set: Union[RoofCeilingConstructionSetAbridged, None] = Field(
        default=None,
        description='A RoofCeilingConstructionSetAbridged object for this '
        'ConstructionSet.'
    )

    aperture_set: Union[ApertureConstructionSetAbridged, None] = Field(
        default=None,
        description='A ApertureConstructionSetAbridged object for this ConstructionSet.'
    )

    door_set: Union[DoorConstructionSetAbridged, None] = Field(
        default=None,
        description='A DoorConstructionSetAbridged object for this ConstructionSet.'
    )

    shade_construction: Union[str, None] = Field(
        default=None,
        min_length=1,
        max_length=100,
        description='The identifier of a ShadeConstruction to set the reflectance '
        'properties of all outdoor shades of all objects to which this '
        'ConstructionSet is assigned.'
    )

    air_boundary_construction: Union[str, None] = Field(
        default=None,
        min_length=1,
        max_length=100,
        description='The identifier of an AirBoundaryConstruction or OpaqueConstruction '
        'to set the properties of Faces with an AirBoundary type.'
    )


class ConstructionSet(ConstructionSetAbridged):
    """A set of constructions for different surface types and boundary conditions."""

    type: Literal['ConstructionSet'] = 'ConstructionSet'

    wall_set: Union[WallConstructionSet, None] = Field(
        default=None,
        description='A WallConstructionSet object for this ConstructionSet.'
    )

    floor_set: Union[FloorConstructionSet, None] = Field(
        default=None,
        description='A FloorConstructionSet object for this ConstructionSet.'
    )

    roof_ceiling_set: Union[RoofCeilingConstructionSet, None] = Field(
        default=None,
        description='A RoofCeilingConstructionSet object for this ConstructionSet.'
    )

    aperture_set: Union[ApertureConstructionSet, None] = Field(
        default=None,
        description='A ApertureConstructionSet object for this ConstructionSet.'
    )

    door_set: Union[DoorConstructionSet, None] = Field(
        default=None,
        description='A DoorConstructionSet object for this ConstructionSet.'
    )

    shade_construction: Union[ShadeConstruction, None] = Field(
        default=None,
        description='A ShadeConstruction to set the reflectance '
        'properties of all outdoor shades of all objects to which this '
        'ConstructionSet is assigned.'
    )

    air_boundary_construction: Union[
        AirBoundaryConstruction, OpaqueConstruction, None
    ] = Field(
        default=None,
        description='An AirBoundaryConstruction or OpaqueConstruction to set '
        'the properties of Faces with an AirBoundary type.'
    )



"""Objects used as alternatives to numerical properties."""
from typing import Literal
from ._base import NoExtraBaseModel


class NoLimit(NoExtraBaseModel):

    type: Literal['NoLimit'] = 'NoLimit'


class Autocalculate(NoExtraBaseModel):

    type: Literal['Autocalculate'] = 'Autocalculate'


class Autosize(NoExtraBaseModel):

    type: Literal['Autosize'] = 'Autosize'



"""Programtype Schema"""
from pydantic import Field
from typing import Union

from ._base import IDdEnergyBaseModel
from .load import PeopleAbridged, LightingAbridged, ElectricEquipmentAbridged, \
    GasEquipmentAbridged, ServiceHotWaterAbridged, \
    InfiltrationAbridged, VentilationAbridged, SetpointAbridged, \
    People, Lighting, ElectricEquipment, GasEquipment, ServiceHotWater, \
    Infiltration, Ventilation, Setpoint
from typing import Literal


class ProgramTypeAbridged(IDdEnergyBaseModel):

    type: Literal['ProgramTypeAbridged'] = 'ProgramTypeAbridged'

    people: Union[PeopleAbridged, None] = Field(
        default=None,
        description='People to describe the occupancy of the program. If None, '
        'no occupancy will be assumed for the program.'
    )

    lighting: Union[LightingAbridged, None] = Field(
        default=None,
        description='Lighting to describe the lighting usage of the program. '
        'If None, no lighting will be assumed for the program.'
    )

    electric_equipment: Union[ElectricEquipmentAbridged, None] = Field(
        default=None,
        description='ElectricEquipment to describe the usage of electric equipment '
        'within the program. If None, no electric equipment will be assumed.'
    )

    gas_equipment: Union[GasEquipmentAbridged, None] = Field(
        default=None,
        description='GasEquipment to describe the usage of gas equipment '
        'within the program. If None, no gas equipment will be assumed.'
    )

    service_hot_water: Union[ServiceHotWaterAbridged, None] = Field(
        default=None,
        description='ServiceHotWater object to describe the usage of hot water '
        'within the program. If None, no hot water will be assumed.'
    )

    infiltration: Union[InfiltrationAbridged, None] = Field(
        default=None,
        description='Infiltration to describe the outdoor air leakage of '
        'the program. If None, no infiltration will be assumed for the program.'
    )

    ventilation: Union[VentilationAbridged, None] = Field(
        default=None,
        description='Ventilation to describe the minimum outdoor air requirement '
        'of the program. If None, no ventilation requirement will be assumed.'
    )

    setpoint: Union[SetpointAbridged, None] = Field(
        default=None,
        description='Setpoint object to describe the temperature and humidity setpoints '
        'of the program.  If None, the ProgramType cannot be assigned to a Room '
        'that is conditioned.'
    )


class ProgramType(ProgramTypeAbridged):

    type: Literal['ProgramType'] = 'ProgramType'

    people: Union[People, None] = Field(
        default=None,
        description='People to describe the occupancy of the program. If None, '
        'no occupancy will be assumed for the program.'
    )

    lighting: Union[Lighting, None] = Field(
        default=None,
        description='Lighting to describe the lighting usage of the program. '
        'If None, no lighting will be assumed for the program.'
    )

    electric_equipment: Union[ElectricEquipment, None] = Field(
        default=None,
        description='ElectricEquipment to describe the usage of electric equipment '
        'within the program. If None, no electric equipment will be assumed.'
    )

    gas_equipment: Union[GasEquipment, None] = Field(
        default=None,
        description='GasEquipment to describe the usage of gas equipment '
        'within the program. If None, no gas equipment will be assumed.'
    )

    service_hot_water: Union[ServiceHotWater, None] = Field(
        default=None,
        description='ServiceHotWater object to describe the usage of hot water '
        'within the program. If None, no hot water will be assumed.'
    )

    infiltration: Union[Infiltration, None] = Field(
        default=None,
        description='Infiltration to describe the outdoor air leakage of '
        'the program. If None, no infiltration will be assumed for the program.'
    )

    ventilation: Union[Ventilation, None] = Field(
        default=None,
        description='Ventilation to describe the minimum outdoor air requirement '
        'of the program. If None, no ventilation requirement will be assumed.'
    )

    setpoint: Union[Setpoint, None] = Field(
        default=None,
        description='Setpoint object to describe the temperature and humidity setpoints '
        'of the program.  If None, the ProgramType cannot be assigned to a Room '
        'that is conditioned.'
    )



"""ModifierSet Schema"""
from pydantic import Field
from typing import Union, Literal
from .._base import NoExtraBaseModel
from ._base import IDdRadianceBaseModel
from .modifier import Plastic, Glass, BSDF, Glow, Light, Trans, Metal, Void, Mirror


class BaseModifierSetAbridged(NoExtraBaseModel):
    """Base class for the abridged modifier sets assigned to Faces."""

    exterior_modifier: Union[str, None] = Field(
        default=None,
        description='Identifier for a radiance modifier object for faces with an '
                    ' Outdoors boundary condition.'
    )

    interior_modifier: Union[str, None] = Field(
        default=None,
        description='Identifier for a radiance modifier object for faces with a '
                    'boundary condition other than Outdoors.'
    )


class WallModifierSetAbridged(BaseModifierSetAbridged):
    """Abridged set containing radiance modifiers needed for a model's Walls."""

    type: Literal['WallModifierSetAbridged'] = 'WallModifierSetAbridged'


class FloorModifierSetAbridged(BaseModifierSetAbridged):
    """Abridged set containing radiance modifiers needed for a model's Floors."""

    type: Literal['FloorModifierSetAbridged'] = 'FloorModifierSetAbridged'


class RoofCeilingModifierSetAbridged(BaseModifierSetAbridged):
    """Abridged set containing radiance modifiers needed for a model's Roofs."""

    type: Literal['RoofCeilingModifierSetAbridged'] = 'RoofCeilingModifierSetAbridged'


class ShadeModifierSetAbridged(BaseModifierSetAbridged):
    """Abridged set containing radiance modifiers needed for a model's Shade."""

    type: Literal['ShadeModifierSetAbridged'] = 'ShadeModifierSetAbridged'


class ApertureModifierSetAbridged(NoExtraBaseModel):
    """Abridged set containing radiance modifiers needed for a model's Apertures."""

    type: Literal['ApertureModifierSetAbridged'] = 'ApertureModifierSetAbridged'

    window_modifier: Union[str, None] = Field(  # type: ignore
        default=None,
        description='Identifier of modifier object for apertures with an Outdoors '
                    'boundary condition, False is_operable property, '
                    'and Wall parent Face.'
    )

    interior_modifier: Union[str, None] = Field(
        default=None,
        description='Identifier of modifier object for apertures with a Surface '
                    'boundary condition.'
    )

    skylight_modifier: Union[str, None] = Field(
        default=None,
        description='Identifier of modifier object for apertures with an Outdoors '
                    'boundary condition, False is_operable property, and a '
                    'RoofCeiling or Floor face type for their parent face.'
    )

    operable_modifier: Union[str, None] = Field(
        default=None,
        description='Identifier of modifier object for apertures with an Outdoors '
                    'boundary condition and a True is_operable property.'
    )


class DoorModifierSetAbridged(BaseModifierSetAbridged):
    """Abridged set containing radiance modifiers needed for a model's Doors."""

    type: Literal['DoorModifierSetAbridged'] = 'DoorModifierSetAbridged'

    interior_glass_modifier: Union[str, None] = Field(
        default=None,
        description='Identifier of modifier object for glass with a Surface '
                    'boundary condition.'
    )

    exterior_glass_modifier: Union[str, None] = Field(
        default=None,
        description='Identifier of modifier object for glass with an Outdoors '
                    'boundary condition.'
    )

    overhead_modifier: Union[str, None] = Field(
        default=None,
        description='Identifier of a modifier object for doors with an '
                    'Outdoors boundary condition and a RoofCeiling or Floor '
                    'face type for their parent face.'
    )


class ModifierSetAbridged(IDdRadianceBaseModel):
    """Abridged set containing all modifiers needed to create a radiance model."""

    type: Literal['ModifierSetAbridged'] = 'ModifierSetAbridged'

    wall_set: Union[WallModifierSetAbridged, None] = Field(
        default=None,
        description='Optional WallModifierSet object for this '
                    'ModifierSet (default: None).'
    )

    floor_set: Union[FloorModifierSetAbridged, None] = Field(
        default=None,
        description='Optional FloorModifierSet object for '
                    'this ModifierSet (default: None).'
    )

    roof_ceiling_set: Union[RoofCeilingModifierSetAbridged, None] = Field(
        default=None,
        description='Optional RoofCeilingModifierSet object for this '
                    'ModifierSet (default: None).'
    )

    aperture_set: Union[ApertureModifierSetAbridged, None] = Field(
        default=None,
        description='Optional ApertureModifierSet object for this '
                    'ModifierSet (default: None).'
    )

    door_set: Union[DoorModifierSetAbridged, None] = Field(
        default=None,
        description='Optional DoorModifierSet object for this '
                    'ModifierSet (default: None).'
    )

    shade_set: Union[ShadeModifierSetAbridged, None] = Field(
        default=None,
        description='Optional ShadeModifierSet object for this '
                    'ModifierSet (default: None).'
    )

    air_boundary_modifier: Union[str, None] = Field(
        default=None,
        description='Optional Modifier to be used for all Faces '
                    'with an AirBoundary face type. If None, it will be the '
                    'honeybee generic air wall modifier.'
    )


ALL_MODIFIERS = Union[Plastic, Glass, BSDF, Glow, Light, Trans, Metal, Void, Mirror, None]


class BaseModifierSet(NoExtraBaseModel):
    """Base class for the modifier sets assigned to Faces."""

    exterior_modifier: ALL_MODIFIERS = Field(
        default=None,
        description='A radiance modifier object for faces with an Outdoors boundary '
                    'condition.'
    )

    interior_modifier: ALL_MODIFIERS = Field(
        default=None,
        description='A radiance modifier object for faces with a boundary condition '
                    'other than Outdoors.'
    )


class WallModifierSet(BaseModifierSet):
    """Set containing radiance modifiers needed for a model's Walls."""

    type: Literal['WallModifierSet'] = 'WallModifierSet'


class FloorModifierSet(BaseModifierSet):
    """Set containing radiance modifiers needed for a model's Floors."""

    type: Literal['FloorModifierSet'] = 'FloorModifierSet'


class RoofCeilingModifierSet(BaseModifierSet):
    """Set containing radiance modifiers needed for a model's roofs."""

    type: Literal['RoofCeilingModifierSet'] = 'RoofCeilingModifierSet'


class ShadeModifierSet(BaseModifierSet):
    """Set containing radiance modifiers needed for a model's Shade."""

    type: Literal['ShadeModifierSet'] = 'ShadeModifierSet'


class ApertureModifierSet(NoExtraBaseModel):
    """Set containing radiance modifiers needed for a model's Apertures."""

    type: Literal['ApertureModifierSet'] = 'ApertureModifierSet'

    window_modifier: ALL_MODIFIERS = Field(
        default=None,
        description='A modifier object for apertures with an Outdoors '
                    'boundary condition, False is_operable property, '
                    'and Wall parent Face.'
    )

    interior_modifier: ALL_MODIFIERS = Field(
        default=None,
        description='A modifier object for apertures with a Surface '
                    'boundary condition.'
    )

    skylight_modifier: ALL_MODIFIERS = Field(
        default=None,
        description='A modifier object for apertures with an Outdoors '
                    'boundary condition, False is_operable property, and a '
                    'RoofCeiling or Floor face type for their parent face.'
    )

    operable_modifier: ALL_MODIFIERS = Field(
        default=None,
        description='A modifier object for apertures with an Outdoors boundary '
                    'condition and a True is_operable property.'
    )


class DoorModifierSet(BaseModifierSet):
    """Set containing radiance modifiers needed for a model's Doors."""

    type: Literal['DoorModifierSet'] = 'DoorModifierSet'

    interior_glass_modifier: ALL_MODIFIERS = Field(
        default=None,
        description='A modifier object for glass with a Surface boundary condition.'
    )

    exterior_glass_modifier: ALL_MODIFIERS = Field(
        default=None,
        description='A modifier object for glass with an Outdoors boundary condition.'
    )

    overhead_modifier: ALL_MODIFIERS = Field(
        default=None,
        description='A window modifier object for doors with an Outdoors boundary '
                    'condition and a RoofCeiling or Floor face type for their '
                    'parent face.'
    )


class ModifierSet(IDdRadianceBaseModel):
    """Set containing all radiance modifiers needed to create a radiance model."""

    type: Literal['ModifierSet'] = 'ModifierSet'

    wall_set: Union[WallModifierSet, None] = Field(
        default=None,
        description='An optional WallModifierSet object for this ModifierSet. '
                    '(default: None).'
    )

    floor_set: Union[FloorModifierSet, None] = Field(
        default=None,
        description='An optional FloorModifierSet object for this ModifierSet. '
                    '(default: None).'
    )

    roof_ceiling_set: Union[RoofCeilingModifierSet, None] = Field(
        default=None,
        description='An optional RoofCeilingModifierSet object for this ModifierSet. '
                    '(default: None).'
    )

    aperture_set: Union[ApertureModifierSet, None] = Field(
        default=None,
        description='An optional ApertureModifierSet object for this ModifierSet. '
                    '(default: None).'
    )

    door_set: Union[DoorModifierSet, None] = Field(
        default=None,
        description='An optional DoorModifierSet object for this ModifierSet. '
                    '(default: None).'
    )

    shade_set: Union[ShadeModifierSet, None] = Field(
        default=None,
        description='An optional ShadeModifierSet object for this ModifierSet. '
                    '(default: None).'
    )

    air_boundary_modifier: ALL_MODIFIERS = Field(
        default=None,
        description='An optional Modifier to be used for all Faces with an AirBoundary '
                    'face type. If None, it will be the honeybee generic air wall '
                    'modifier.'
    )
